@@ -38,6 +38,16 @@ form input::
     <input type="hidden" name="x_delete_at" value="<unix-timestamp>" />
     <input type="hidden" name="x_delete_after" value="<seconds>" />
 
+If you want to specify the content type or content encoding of the files you
+can set content-encoding or content-type by adding them to the form input::
+
+    <input type="hidden" name="content-type" value="text/html" />
+    <input type="hidden" name="content-encoding" value="gzip" />
+
+The above example applies these parameters to all uploaded files. You can also
+set the content-type and content-encoding on a per-file basis by adding the
+parameters to each part of the upload.
+
 The <swift-url> is the URL of the Swift destination, such as::
 
     https://swift-cluster.example.com/v1/AUTH_account/container/object_prefix
@@ -116,14 +126,16 @@ import hmac
 from hashlib import sha1
 from time import time
 
+import six
 from six.moves.urllib.parse import quote
+
 from swift.common.exceptions import MimeInvalid
 from swift.common.middleware.tempurl import get_tempurl_keys_from_metadata
 from swift.common.utils import streq_const_time, register_swift_info, \
     parse_content_disposition, parse_mime_headers, \
-    iter_multipart_mime_documents
+    iter_multipart_mime_documents, reiterate, close_if_possible
 from swift.common.wsgi import make_pre_authed_env
-from swift.common.swob import HTTPUnauthorized
+from swift.common.swob import HTTPUnauthorized, wsgi_to_str
 from swift.proxy.controllers.base import get_account_info, get_container_info
 
 
@@ -219,7 +231,7 @@ class FormPost(object):
                     start_response(status, headers)
                     return [body]
             except MimeInvalid:
-                body = 'FormPost: invalid starting boundary'
+                body = b'FormPost: invalid starting boundary'
                 start_response(
                     '400 Bad Request',
                     (('Content-Type', 'text/plain'),
@@ -227,6 +239,8 @@ class FormPost(object):
                 return [body]
             except (FormInvalid, EOFError) as err:
                 body = 'FormPost: %s' % err
+                if six.PY3:
+                    body = body.encode('utf-8')
                 start_response(
                     '400 Bad Request',
                     (('Content-Type', 'text/plain'),
@@ -248,6 +262,8 @@ class FormPost(object):
         :returns: status_line, headers_list, body
         """
         keys = self._get_keys(env)
+        if six.PY3:
+            boundary = boundary.encode('utf-8')
         status = message = ''
         attributes = {}
         subheaders = []
@@ -270,12 +286,15 @@ class FormPost(object):
                 if 'content-type' not in attributes and 'content-type' in hdrs:
                     attributes['content-type'] = \
                         hdrs['Content-Type'] or 'application/octet-stream'
-                status, subheaders, message = \
+                if 'content-encoding' not in attributes and \
+                        'content-encoding' in hdrs:
+                    attributes['content-encoding'] = hdrs['Content-Encoding']
+                status, subheaders = \
                     self._perform_subrequest(env, attributes, fp, keys)
                 if not status.startswith('2'):
                     break
             else:
-                data = ''
+                data = b''
                 mxln = MAX_VALUE_LENGTH
                 while mxln:
                     chunk = fp.read(mxln)
@@ -285,6 +304,8 @@ class FormPost(object):
                     data += chunk
                 while fp.read(READ_CHUNK_SIZE):
                     pass
+                if six.PY3:
+                    data = data.decode('utf-8')
                 if 'name' in attrs:
                     attributes[attrs['name'].lower()] = data.rstrip('\r\n--')
         if not status:
@@ -301,6 +322,8 @@ class FormPost(object):
                 body = status + '\r\nFormPost: ' + message.title()
             headers.extend([('Content-Type', 'text/plain'),
                             ('Content-Length', len(body))])
+            if six.PY3:
+                body = body.encode('utf-8')
             return status, headers, body
         status = status.split(' ', 1)[0]
         if '?' in redirect:
@@ -310,6 +333,8 @@ class FormPost(object):
         redirect += 'status=%s&message=%s' % (quote(status), quote(message))
         body = '<html><body><p><a href="%s">' \
                'Click to continue...</a></p></body></html>' % redirect
+        if six.PY3:
+            body = body.encode('utf-8')
         headers.extend(
             [('Location', redirect), ('Content-Length', str(len(body)))])
         return '303 See Other', headers, body
@@ -323,7 +348,7 @@ class FormPost(object):
         :param attributes: dict of the attributes of the form so far.
         :param fp: The file-like object containing the request body.
         :param keys: The account keys to validate the signature with.
-        :returns: (status_line, headers_list, message)
+        :returns: (status_line, headers_list)
         """
         if not keys:
             raise FormUnauthorized('invalid signature')
@@ -357,22 +382,27 @@ class FormPost(object):
         if 'content-type' in attributes:
             subenv['CONTENT_TYPE'] = \
                 attributes['content-type'] or 'application/octet-stream'
-        elif 'CONTENT_TYPE' in subenv:
-            del subenv['CONTENT_TYPE']
+        if 'content-encoding' in attributes:
+            subenv['HTTP_CONTENT_ENCODING'] = attributes['content-encoding']
         try:
             if int(attributes.get('expires') or 0) < time():
                 raise FormUnauthorized('form expired')
         except ValueError:
             raise FormInvalid('expired not an integer')
         hmac_body = '%s\n%s\n%s\n%s\n%s' % (
-            orig_env['PATH_INFO'],
+            wsgi_to_str(orig_env['PATH_INFO']),
             attributes.get('redirect') or '',
             attributes.get('max_file_size') or '0',
             attributes.get('max_file_count') or '0',
             attributes.get('expires') or '0')
+        if six.PY3:
+            hmac_body = hmac_body.encode('utf-8')
 
         has_valid_sig = False
         for key in keys:
+            # Encode key like in swift.common.utls.get_hmac.
+            if not isinstance(key, six.binary_type):
+                key = key.encode('utf8')
             sig = hmac.new(key, hmac_body, sha1).hexdigest()
             if streq_const_time(sig, (attributes.get('signature') or
                                       'invalid')):
@@ -392,12 +422,10 @@ class FormPost(object):
             substatus[0] = status
             subheaders[0] = headers
 
-        i = iter(self.app(subenv, _start_response))
-        try:
-            next(i)
-        except StopIteration:
-            pass
-        return substatus[0], subheaders[0], ''
+        # reiterate to ensure the response started,
+        # but drop any data on the floor
+        close_if_possible(reiterate(self.app(subenv, _start_response)))
+        return substatus[0], subheaders[0]
 
     def _get_keys(self, env):
         """
