@@ -30,12 +30,20 @@ symlink, the header ``X-Symlink-Target-Account: <account>`` must be included.
 If omitted, it is inserted automatically with the account of the symlink
 object in the PUT request process.
 
-Symlinks must be zero-byte objects. Attempting to PUT a symlink
-with a non-empty request body will result in a 400-series error. Also, POST
-with X-Symlink-Target header always results in a 400-series error. The target
-object need not exist at symlink creation time. It is suggested to set the
-``Content-Type`` of symlink objects to a distinct value such as
-``application/symlink``.
+Symlinks must be zero-byte objects. Attempting to PUT a symlink with a
+non-empty request body will result in a 400-series error. Also, POST with
+``X-Symlink-Target`` header always results in a 400-series error. The target
+object need not exist at symlink creation time.
+
+Clients may optionally include a ``X-Symlink-Target-Etag: <etag>`` header
+during the PUT. If present, this will create a "static symlink" instead of a
+"dynamic symlink".  Static symlinks point to a specific object rather than a
+specific name.  They do this by using the value set in their
+``X-Symlink-Target-Etag`` header when created to verify it still matches the
+ETag of the object they're pointing at on a GET.  In contrast to a dynamic
+symlink the target object referenced in the ``X-Symlink-Target`` header must
+exist and its ETag must match the ``X-Symlink-Target-Etag`` or the symlink
+creation will return a client error.
 
 A GET/HEAD request to a symlink will result in a request to the target
 object referenced by the symlink's ``X-Symlink-Target-Account`` and
@@ -45,12 +53,22 @@ GET/HEAD request to a symlink with the query parameter ``?symlink=get`` will
 result in the request targeting the symlink itself.
 
 A symlink can point to another symlink. Chained symlinks will be traversed
-until target is not a symlink. If the number of chained symlinks exceeds the
-limit ``symloop_max`` an error response will be produced. The value of
+until the target is not a symlink. If the number of chained symlinks exceeds
+the limit ``symloop_max`` an error response will be produced. The value of
 ``symloop_max`` can be defined in the symlink config section of
 `proxy-server.conf`. If not specified, the default ``symloop_max`` value is 2.
 If a value less than 1 is specified, the default value will be used.
 
+If a static symlink (i.e. a symlink created with a ``X-Symlink-Target-Etag``
+header) targets another static symlink, both of the ``X-Symlink-Target-Etag``
+headers must match the target object for the GET to succeed.  If a static
+symlink targets a dynamic symlink (i.e. a symlink created without a
+``X-Symlink-Target-Etag`` header) then the ``X-Symlink-Target-Etag`` header of
+the static symlink must be the Etag of the zero-byte object.  If a symlink with
+a ``X-Symlink-Target-Etag`` targets a large object manifest it must match the
+ETag of the manifest (e.g. the ETag as returned by ``multipart-manifest=get``
+or value in the ``X-Manifest-Etag`` header).
+
 A HEAD/GET request to a symlink object behaves as a normal HEAD/GET request
 to the target object. Therefore issuing a HEAD request to the symlink will
 return the target metadata, and issuing a GET request to the symlink will
@@ -58,13 +76,22 @@ return the data and metadata of the target object. To return the symlink
 metadata (with its empty body) a GET/HEAD request with the ``?symlink=get``
 query parameter must be sent to a symlink object.
 
-A POST request to a symlink will result in a 307 TemporaryRedirect response.
+A POST request to a symlink will result in a 307 Temporary Redirect response.
 The response will contain a ``Location`` header with the path of the target
 object as the value. The request is never redirected to the target object by
 Swift. Nevertheless, the metadata in the POST request will be applied to the
 symlink because object servers cannot know for sure if the current object is a
 symlink or not in eventual consistency.
 
+A symlink's ``Content-Type`` is completely independent from its target.  As a
+convenience Swift will automatically set the ``Content-Type`` on a symlink PUT
+if not explicitly set by the client.  If the client sends a
+``X-Symlink-Target-Etag`` Swift will set the symlink's ``Content-Type`` to that
+of the target, otherwise it will be set to ``application/symlink``.  You can
+review a symlink's ``Content-Type`` using the ``?symlink=get`` interface.  You
+can change a symlink's ``Content-Type`` using a POST request.  The symlink's
+``Content-Type`` will appear in the container listing.
+
 A DELETE request to a symlink will delete the symlink itself. The target
 object will not be deleted.
 
@@ -73,7 +100,7 @@ will copy the target object. The same request to a symlink with the query
 parameter ``?symlink=get`` will copy the symlink itself.
 
 An OPTIONS request to a symlink will respond with the options for the symlink
-only, the request will not be redirected to the target object. Please note that
+only; the request will not be redirected to the target object. Please note that
 if the symlink's target object is in another container with CORS settings, the
 response will not reflect the settings.
 
@@ -82,7 +109,8 @@ will result in a 400-series error. The GET/HEAD tempurls honor the scope of
 the tempurl key. Container tempurl will only work on symlinks where the target
 container is the same as the symlink. In case a symlink targets an object
 in a different container, a GET/HEAD request will result in a 401 Unauthorized
-error. The account level tempurl will allow cross container symlinks.
+error. The account level tempurl will allow cross-container symlinks, but not
+cross-account symlinks.
 
 If a symlink object is overwritten while it is in a versioned container, the
 symlink object itself is versioned, not the referenced object.
@@ -91,8 +119,19 @@ A GET request with query parameter ``?format=json`` to a container which
 contains symlinks will respond with additional information ``symlink_path``
 for each symlink object in the container listing. The ``symlink_path`` value
 is the target path of the symlink. Clients can differentiate symlinks and
-other objects by this function. Note that responses of any other format
-(e.g.``?format=xml``) won't include ``symlink_path`` info.
+other objects by this function. Note that responses in any other format
+(e.g. ``?format=xml``) won't include ``symlink_path`` info.  If a
+``X-Symlink-Target-Etag`` header was included on the symlink, JSON container
+listings will include that value in a ``symlink_etag`` key and the target
+object's ``Content-Length`` will be included in the key ``symlink_bytes``.
+
+If a static symlink targets a static large object manifest it will carry
+forward the SLO's size and slo_etag in the container listing using the
+``symlink_bytes`` and ``slo_etag`` keys.  However, manifests created before
+swift v2.12.0 (released Dec 2016) do not contain enough metadata to propagate
+the extra SLO information to the listing.  Clients may recreate the manifest
+(COPY w/ ``?multipart-manfiest=get``) before creating a static symlink to add
+the requisite metadata.
 
 Errors
 
@@ -105,7 +144,10 @@ Errors
 * GET/HEAD traversing more than ``symloop_max`` chained symlinks will
   produce a 409 Conflict error.
 
-* POSTs will produce a 307 TemporaryRedirect error.
+* PUT/GET/HEAD on a symlink that inclues a ``X-Symlink-Target-Etag`` header
+  that does not match the target will poduce a 409 Conflict error.
+
+* POSTs will produce a 307 Temporary Redirect error.
 
 ----------
 Deployment
@@ -160,15 +202,15 @@ import os
 from cgi import parse_header
 
 from swift.common.utils import get_logger, register_swift_info, split_path, \
-    MD5_OF_EMPTY_STRING, closing_if_possible
+    MD5_OF_EMPTY_STRING, close_if_possible, closing_if_possible
 from swift.common.constraints import check_account_format
 from swift.common.wsgi import WSGIContext, make_subrequest
 from swift.common.request_helpers import get_sys_meta_prefix, \
-    check_path_header
+    check_path_header, get_container_update_override_key
 from swift.common.swob import Request, HTTPBadRequest, HTTPTemporaryRedirect, \
     HTTPException, HTTPConflict, HTTPPreconditionFailed, wsgi_quote, \
     wsgi_unquote
-from swift.common.http import is_success
+from swift.common.http import is_success, HTTP_NOT_FOUND
 from swift.common.exceptions import LinkIterError
 from swift.common.header_key_dict import HeaderKeyDict
 
@@ -176,22 +218,33 @@ DEFAULT_SYMLOOP_MAX = 2
 # Header values for symlink target path strings will be quoted values.
 TGT_OBJ_SYMLINK_HDR = 'x-symlink-target'
 TGT_ACCT_SYMLINK_HDR = 'x-symlink-target-account'
+TGT_ETAG_SYMLINK_HDR = 'x-symlink-target-etag'
+TGT_BYTES_SYMLINK_HDR = 'x-symlink-target-bytes'
 TGT_OBJ_SYSMETA_SYMLINK_HDR = get_sys_meta_prefix('object') + 'symlink-target'
 TGT_ACCT_SYSMETA_SYMLINK_HDR = \
     get_sys_meta_prefix('object') + 'symlink-target-account'
+TGT_ETAG_SYSMETA_SYMLINK_HDR = \
+    get_sys_meta_prefix('object') + 'symlink-target-etag'
+TGT_BYTES_SYSMETA_SYMLINK_HDR = \
+    get_sys_meta_prefix('object') + 'symlink-target-bytes'
 
 
-def _check_symlink_header(req):
+def _validate_and_prep_request_headers(req):
     """
-    Validate that the value from x-symlink-target header is
-    well formatted. We assume the caller ensures that
+    Validate that the value from x-symlink-target header is well formatted
+    and that the x-symlink-target-etag header (if present) does not contain
+    problematic characters. We assume the caller ensures that
     x-symlink-target header is present in req.headers.
 
     :param req: HTTP request object
+    :returns: a tuple, the full versioned path to the object (as a WSGI string)
+              and the X-Symlink-Target-Etag header value which may be None
     :raise: HTTPPreconditionFailed if x-symlink-target value
             is not well formatted.
     :raise: HTTPBadRequest if the x-symlink-target value points to the request
             path.
+    :raise: HTTPBadRequest if the x-symlink-target-etag value contains
+            a semicolon, double-quote, or backslash.
     """
     # N.B. check_path_header doesn't assert the leading slash and
     # copy middleware may accept the format. In the symlink, API
@@ -228,43 +281,48 @@ def _check_symlink_header(req):
         raise HTTPBadRequest(
             body='Symlink cannot target itself',
             request=req, content_type='text/plain')
+    etag = req.headers.get(TGT_ETAG_SYMLINK_HDR, None)
+    if etag and any(c in etag for c in ';"\\'):
+        # See cgi.parse_header for why the above chars are problematic
+        raise HTTPBadRequest(
+            body='Bad %s format' % TGT_ETAG_SYMLINK_HDR.title(),
+            request=req, content_type='text/plain')
+    if not (etag or req.headers.get('Content-Type')):
+        req.headers['Content-Type'] = 'application/symlink'
+    return '/v1/%s/%s/%s' % (account, container, obj), etag
 
 
 def symlink_usermeta_to_sysmeta(headers):
     """
-    Helper function to translate from X-Symlink-Target and
-    X-Symlink-Target-Account to X-Object-Sysmeta-Symlink-Target
-    and X-Object-Sysmeta-Symlink-Target-Account.
+    Helper function to translate from client-facing X-Symlink-* headers
+    to cluster-facing X-Object-Sysmeta-Symlink-* headers.
 
     :param headers: request headers dict. Note that the headers dict
         will be updated directly.
     """
     # To preseve url-encoded value in the symlink header, use raw value
-    if TGT_OBJ_SYMLINK_HDR in headers:
-        headers[TGT_OBJ_SYSMETA_SYMLINK_HDR] = headers.pop(
-            TGT_OBJ_SYMLINK_HDR)
-
-    if TGT_ACCT_SYMLINK_HDR in headers:
-        headers[TGT_ACCT_SYSMETA_SYMLINK_HDR] = headers.pop(
-            TGT_ACCT_SYMLINK_HDR)
+    for user_hdr, sysmeta_hdr in (
+            (TGT_OBJ_SYMLINK_HDR, TGT_OBJ_SYSMETA_SYMLINK_HDR),
+            (TGT_ACCT_SYMLINK_HDR, TGT_ACCT_SYSMETA_SYMLINK_HDR)):
+        if user_hdr in headers:
+            headers[sysmeta_hdr] = headers.pop(user_hdr)
 
 
 def symlink_sysmeta_to_usermeta(headers):
     """
-    Helper function to translate from X-Object-Sysmeta-Symlink-Target and
-    X-Object-Sysmeta-Symlink-Target-Account to X-Symlink-Target and
-    X-Sysmeta-Symlink-Target-Account
+    Helper function to translate from cluster-facing
+    X-Object-Sysmeta-Symlink-* headers to client-facing X-Symlink-* headers.
 
     :param headers: request headers dict. Note that the headers dict
         will be updated directly.
     """
-    if TGT_OBJ_SYSMETA_SYMLINK_HDR in headers:
-        headers[TGT_OBJ_SYMLINK_HDR] = headers.pop(
-            TGT_OBJ_SYSMETA_SYMLINK_HDR)
-
-    if TGT_ACCT_SYSMETA_SYMLINK_HDR in headers:
-        headers[TGT_ACCT_SYMLINK_HDR] = headers.pop(
-            TGT_ACCT_SYSMETA_SYMLINK_HDR)
+    for user_hdr, sysmeta_hdr in (
+            (TGT_OBJ_SYMLINK_HDR, TGT_OBJ_SYSMETA_SYMLINK_HDR),
+            (TGT_ACCT_SYMLINK_HDR, TGT_ACCT_SYSMETA_SYMLINK_HDR),
+            (TGT_ETAG_SYMLINK_HDR, TGT_ETAG_SYSMETA_SYMLINK_HDR),
+            (TGT_BYTES_SYMLINK_HDR, TGT_BYTES_SYSMETA_SYMLINK_HDR)):
+        if sysmeta_hdr in headers:
+            headers[user_hdr] = headers.pop(sysmeta_hdr)
 
 
 class SymlinkContainerContext(WSGIContext):
@@ -308,9 +366,10 @@ class SymlinkContainerContext(WSGIContext):
 
     def _extract_symlink_path_json(self, obj_dict, swift_version, account):
         """
-        Extract the symlink path from the hash value
-        :return: object dictionary with additional key:value pair if object
-                 is a symlink. The new key is symlink_path.
+        Extract the symlink info from the hash value
+        :return: object dictionary with additional key:value pairs when object
+                 is a symlink. i.e. new symlink_path, symlink_etag and
+                 symlink_bytes keys
         """
         if 'hash' in obj_dict:
             hash_value, meta = parse_header(obj_dict['hash'])
@@ -321,6 +380,10 @@ class SymlinkContainerContext(WSGIContext):
                     target = meta[key]
                 elif key == 'symlink_target_account':
                     account = meta[key]
+                elif key == 'symlink_target_etag':
+                    obj_dict['symlink_etag'] = meta[key]
+                elif key == 'symlink_target_bytes':
+                    obj_dict['symlink_bytes'] = int(meta[key])
                 else:
                     # make sure to add all other (key, values) back in place
                     obj_dict['hash'] += '; %s=%s' % (key, meta[key])
@@ -370,10 +433,11 @@ class SymlinkObjectContext(WSGIContext):
         except LinkIterError:
             errmsg = 'Too many levels of symbolic links, ' \
                      'maximum allowed is %d' % self.symloop_max
-            raise HTTPConflict(
-                body=errmsg, request=req, content_type='text/plain')
+            raise HTTPConflict(body=errmsg, request=req,
+                               content_type='text/plain')
 
-    def _recursive_get_head(self, req):
+    def _recursive_get_head(self, req, target_etag=None,
+                            follow_softlinks=True):
         resp = self._app_call(req.environ)
 
         def build_traversal_req(symlink_target):
@@ -396,14 +460,35 @@ class SymlinkObjectContext(WSGIContext):
 
         symlink_target = self._response_header_value(
             TGT_OBJ_SYSMETA_SYMLINK_HDR)
-        if symlink_target:
+        resp_etag = self._response_header_value(
+            TGT_ETAG_SYSMETA_SYMLINK_HDR)
+        if symlink_target and (resp_etag or follow_softlinks):
+            close_if_possible(resp)
+            found_etag = resp_etag or self._response_header_value('etag')
+            if target_etag and target_etag != found_etag:
+                raise HTTPConflict(
+                    body='X-Symlink-Target-Etag headers do not match',
+                    headers={
+                        'Content-Type': 'text/plain',
+                        'Content-Location': self._last_target_path})
             if self._loop_count >= self.symloop_max:
                 raise LinkIterError()
             # format: /<account name>/<container name>/<object name>
             new_req = build_traversal_req(symlink_target)
             self._loop_count += 1
-            return self._recursive_get_head(new_req)
+            return self._recursive_get_head(new_req, target_etag=resp_etag)
         else:
+            final_etag = self._response_header_value('etag')
+            if final_etag and target_etag and target_etag != final_etag:
+                close_if_possible(resp)
+                body = ('Object Etag %r does not match '
+                        'X-Symlink-Target-Etag header %r')
+                raise HTTPConflict(
+                    body=body % (final_etag, target_etag),
+                    headers={
+                        'Content-Type': 'text/plain',
+                        'Content-Location': self._last_target_path})
+
             if self._last_target_path:
                 # Content-Location will be applied only when one or more
                 # symlink recursion occurred.
@@ -417,6 +502,47 @@ class SymlinkObjectContext(WSGIContext):
 
             return resp
 
+    def _validate_etag_and_update_sysmeta(self, req, symlink_target_path,
+                                          etag):
+        # next we'll make sure the E-Tag matches a real object
+        new_req = make_subrequest(
+            req.environ, path=wsgi_quote(symlink_target_path), method='HEAD',
+            swift_source='SYM')
+        self._last_target_path = symlink_target_path
+        resp = self._recursive_get_head(new_req, target_etag=etag,
+                                        follow_softlinks=False)
+        if self._get_status_int() == HTTP_NOT_FOUND:
+            raise HTTPConflict(
+                body='X-Symlink-Target does not exist',
+                headers={
+                    'Content-Type': 'text/plain',
+                    'Content-Location': self._last_target_path})
+        if not is_success(self._get_status_int()):
+            return resp
+        response_headers = HeaderKeyDict(self._response_headers)
+        # carry forward any etag update params (e.g. "slo_etag"), we'll append
+        # symlink_target_* params to this header after this method returns
+        override_header = get_container_update_override_key('etag')
+        if override_header in response_headers and \
+                override_header not in req.headers:
+            sep, params = response_headers[override_header].partition(';')[1:]
+            req.headers[override_header] = MD5_OF_EMPTY_STRING + sep + params
+
+        # It's troublesome that there's so much leakage with SLO
+        if 'X-Object-Sysmeta-Slo-Etag' in response_headers and \
+                override_header not in req.headers:
+            req.headers[override_header] = '%s; slo_etag=%s' % (
+                MD5_OF_EMPTY_STRING,
+                response_headers['X-Object-Sysmeta-Slo-Etag'])
+        req.headers[TGT_BYTES_SYSMETA_SYMLINK_HDR] = (
+            response_headers.get('x-object-sysmeta-slo-size') or
+            response_headers['Content-Length'])
+
+        req.headers[TGT_ETAG_SYSMETA_SYMLINK_HDR] = etag
+
+        if not req.headers.get('Content-Type'):
+            req.headers['Content-Type'] = response_headers['Content-Type']
+
     def handle_put(self, req):
         """
         Handle put request when it contains X-Symlink-Target header.
@@ -425,13 +551,23 @@ class SymlinkObjectContext(WSGIContext):
         :param req: HTTP PUT object request
         :returns: Response Iterator
         """
-        if req.content_length != 0:
+        if req.content_length is None:
+            has_body = (req.body_file.read(1) != b'')
+        else:
+            has_body = (req.content_length != 0)
+        if has_body:
             raise HTTPBadRequest(
                 body='Symlink requests require a zero byte body',
                 request=req,
                 content_type='text/plain')
 
-        _check_symlink_header(req)
+        symlink_target_path, etag = _validate_and_prep_request_headers(req)
+        if etag:
+            resp = self._validate_etag_and_update_sysmeta(
+                req, symlink_target_path, etag)
+            if resp is not None:
+                return resp
+        # N.B. TGT_ETAG_SYMLINK_HDR was converted as part of verifying it
         symlink_usermeta_to_sysmeta(req.headers)
         # Store info in container update that this object is a symlink.
         # We have a design decision to use etag space to store symlink info for
@@ -441,17 +577,31 @@ class SymlinkObjectContext(WSGIContext):
         # listing result for clients.
         # To create override etag easily, we have a constraint that the symlink
         # must be 0 byte so we can add etag of the empty string + symlink info
-        # here, simply. Note that this override etag may be encrypted in the
-        # container db by encryption middleware.
+        # here, simply (if no other override etag was provided). Note that this
+        # override etag may be encrypted in the container db by encryption
+        # middleware.
+
         etag_override = [
-            MD5_OF_EMPTY_STRING,
+            req.headers.get(get_container_update_override_key('etag'),
+                            MD5_OF_EMPTY_STRING),
             'symlink_target=%s' % req.headers[TGT_OBJ_SYSMETA_SYMLINK_HDR]
         ]
         if TGT_ACCT_SYSMETA_SYMLINK_HDR in req.headers:
             etag_override.append(
                 'symlink_target_account=%s' %
                 req.headers[TGT_ACCT_SYSMETA_SYMLINK_HDR])
-        req.headers['X-Object-Sysmeta-Container-Update-Override-Etag'] = \
+        if TGT_ETAG_SYSMETA_SYMLINK_HDR in req.headers:
+            # if _validate_etag_and_update_sysmeta or a middleware sets
+            # TGT_ETAG_SYSMETA_SYMLINK_HDR then they need to also set
+            # TGT_BYTES_SYSMETA_SYMLINK_HDR.  If they forget, they get a
+            # KeyError traceback and client gets a ServerError
+            etag_override.extend([
+                'symlink_target_etag=%s' %
+                req.headers[TGT_ETAG_SYSMETA_SYMLINK_HDR],
+                'symlink_target_bytes=%s' %
+                req.headers[TGT_BYTES_SYSMETA_SYMLINK_HDR],
+            ])
+        req.headers[get_container_update_override_key('etag')] = \
             '; '.join(etag_override)
 
         return self._app_call(req.environ)
@@ -491,11 +641,16 @@ class SymlinkObjectContext(WSGIContext):
                 TGT_ACCT_SYSMETA_SYMLINK_HDR) or wsgi_quote(account)
             location_hdr = os.path.join(
                 '/', version, target_acc, tgt_co)
+            headers = {'location': location_hdr}
+            tgt_etag = self._response_header_value(
+                TGT_ETAG_SYSMETA_SYMLINK_HDR)
+            if tgt_etag:
+                headers[TGT_ETAG_SYMLINK_HDR] = tgt_etag
             req.environ['swift.leave_relative_location'] = True
             errmsg = 'The requested POST was applied to a symlink. POST ' +\
                      'directly to the target to apply requested metadata.'
             raise HTTPTemporaryRedirect(
-                body=errmsg, headers={'location': location_hdr})
+                body=errmsg, headers=headers)
         else:
             return resp
 
@@ -508,10 +663,7 @@ class SymlinkObjectContext(WSGIContext):
         :returns: Response Iterator after start_response has been called
         """
         if req.method in ('GET', 'HEAD'):
-            # if GET request came from versioned writes, then it should get
-            # the symlink only, not the referenced target
-            if req.params.get('symlink') == 'get' or \
-                    req.environ.get('swift.source') == 'VW':
+            if req.params.get('symlink') == 'get':
                 resp = self.handle_get_head_symlink(req)
             else:
                 resp = self.handle_get_head(req)
@@ -578,7 +730,7 @@ def filter_factory(global_conf, **local_conf):
     symloop_max = int(conf.get('symloop_max', DEFAULT_SYMLOOP_MAX))
     if symloop_max < 1:
         symloop_max = int(DEFAULT_SYMLOOP_MAX)
-    register_swift_info('symlink', symloop_max=symloop_max)
+    register_swift_info('symlink', symloop_max=symloop_max, static_links=True)
 
     def symlink_mw(app):
         return SymlinkMiddleware(app, conf, symloop_max)
