@@ -44,6 +44,8 @@ from swift.common.wsgi import make_subrequest
 
 
 OBJECT_TRANSIENT_SYSMETA_PREFIX = 'x-object-transient-sysmeta-'
+OBJECT_SYSMETA_CONTAINER_UPDATE_OVERRIDE_PREFIX = \
+    'x-object-sysmeta-container-update-override-'
 
 
 def get_param(req, name, default=None):
@@ -260,6 +262,17 @@ def get_object_transient_sysmeta(key):
     return '%s%s' % (OBJECT_TRANSIENT_SYSMETA_PREFIX, key)
 
 
+def get_container_update_override_key(key):
+    """
+    Returns the full X-Object-Sysmeta-Container-Update-Override-* header key.
+
+    :param key: the key you want to override in the container update
+    :returns: the full header key
+    """
+    header = '%s%s' % (OBJECT_SYSMETA_CONTAINER_UPDATE_OVERRIDE_PREFIX, key)
+    return header.title()
+
+
 def remove_items(headers, condition):
     """
     Removes items from a dict whose keys satisfy
