@@ -14,17 +14,19 @@
 # limitations under the License.
 
 import base64
+import binascii
 import hashlib
 from mock import patch
 import os
 import time
 import unittest
-from urllib import quote
+from six.moves.urllib.parse import quote
 
 from swift.common import swob
 from swift.common.swob import Request
 from swift.common.utils import json
 
+from test.unit import FakeMemcache
 from test.unit.common.middleware.s3api import S3ApiTestCase
 from test.unit.common.middleware.s3api.helpers import UnreadableInput
 from swift.common.middleware.s3api.etree import fromstring, tostring
@@ -34,8 +36,9 @@ from test.unit.common.middleware.s3api.test_s3_acl import s3acl
 from swift.common.middleware.s3api.utils import sysmeta_header, mktime, \
     S3Timestamp
 from swift.common.middleware.s3api.s3request import MAX_32BIT_INT
+from swift.proxy.controllers.base import get_cache_key
 
-xml = '<CompleteMultipartUpload>' \
+XML = '<CompleteMultipartUpload>' \
     '<Part>' \
     '<PartNumber>1</PartNumber>' \
     '<ETag>0123456789abcdef0123456789abcdef</ETag>' \
@@ -46,11 +49,11 @@ xml = '<CompleteMultipartUpload>' \
     '</Part>' \
     '</CompleteMultipartUpload>'
 
-objects_template = \
+OBJECTS_TEMPLATE = \
     (('object/X/1', '2014-05-07T19:47:51.592270', '0123456789abcdef', 100),
      ('object/X/2', '2014-05-07T19:47:52.592270', 'fedcba9876543210', 200))
 
-multiparts_template = \
+MULTIPARTS_TEMPLATE = \
     (('object/X', '2014-05-07T19:47:50.592270', 'HASH', 1),
      ('object/X/1', '2014-05-07T19:47:51.592270', '0123456789abcdef', 11),
      ('object/X/2', '2014-05-07T19:47:52.592270', 'fedcba9876543210', 21),
@@ -66,9 +69,9 @@ multiparts_template = \
      ('subdir/object/Z/2', '2014-05-07T19:47:58.592270', 'fedcba9876543210',
       41))
 
-s3_etag = '"%s-2"' % hashlib.md5((
+S3_ETAG = '"%s-2"' % hashlib.md5(binascii.a2b_hex(
     '0123456789abcdef0123456789abcdef'
-    'fedcba9876543210fedcba9876543210').decode('hex')).hexdigest()
+    'fedcba9876543210fedcba9876543210')).hexdigest()
 
 
 class TestS3ApiMultiUpload(S3ApiTestCase):
@@ -83,9 +86,9 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
 
         self.s3api.conf.min_segment_size = 1
 
-        objects = map(lambda item: {'name': item[0], 'last_modified': item[1],
-                                    'hash': item[2], 'bytes': item[3]},
-                      objects_template)
+        objects = [{'name': item[0], 'last_modified': item[1],
+                    'hash': item[2], 'bytes': item[3]}
+                   for item in OBJECTS_TEMPLATE]
         object_list = json.dumps(objects)
 
         self.swift.register('PUT', segment_bucket,
@@ -171,11 +174,11 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
     def _test_bucket_multipart_uploads_GET(self, query=None,
                                            multiparts=None):
         segment_bucket = '/v1/AUTH_test/bucket+segments'
-        objects = multiparts or multiparts_template
-        objects = map(lambda item: {'name': item[0], 'last_modified': item[1],
-                                    'hash': item[2], 'bytes': item[3]},
-                      objects)
-        object_list = json.dumps(objects)
+        objects = multiparts or MULTIPARTS_TEMPLATE
+        objects = [{'name': item[0], 'last_modified': item[1],
+                    'hash': item[2], 'bytes': item[3]}
+                   for item in objects]
+        object_list = json.dumps(objects).encode('ascii')
         self.swift.register('GET', segment_bucket, swob.HTTPOk, {},
                             object_list)
 
@@ -197,7 +200,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(elem.find('MaxUploads').text, '1000')
         self.assertEqual(elem.find('IsTruncated').text, 'false')
         self.assertEqual(len(elem.findall('Upload')), 4)
-        objects = [(o[0], o[1][:-3] + 'Z') for o in multiparts_template]
+        objects = [(o[0], o[1][:-3] + 'Z') for o in MULTIPARTS_TEMPLATE]
         for u in elem.findall('Upload'):
             name = u.find('Key').text + '/' + u.find('UploadId').text
             initiated = u.find('Initiated').text
@@ -538,14 +541,16 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
 
     @patch('swift.common.middleware.s3api.controllers.'
            'multi_upload.unique_id', lambda: 'X')
-    def _test_object_multipart_upload_initiate(self, headers):
+    def _test_object_multipart_upload_initiate(self, headers, cache=None,
+                                               bucket_exists=True):
         headers.update({
             'Authorization': 'AWS test:tester:hmac',
             'Date': self.get_date_header(),
             'x-amz-meta-foo': 'bar',
         })
         req = Request.blank('/bucket/object?uploads',
-                            environ={'REQUEST_METHOD': 'POST'},
+                            environ={'REQUEST_METHOD': 'POST',
+                                     'swift.cache': cache},
                             headers=headers)
         status, headers, body = self.call_s3api(req)
         fromstring(body, 'InitiateMultipartUploadResult')
@@ -555,27 +560,63 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(req_headers.get('X-Object-Meta-Foo'), 'bar')
         self.assertNotIn('Etag', req_headers)
         self.assertNotIn('Content-MD5', req_headers)
-        self.assertEqual([
-            ('HEAD', '/v1/AUTH_test/bucket'),
-            ('PUT', '/v1/AUTH_test/bucket+segments'),
-            ('HEAD', '/v1/AUTH_test'),
-            ('HEAD', '/v1/AUTH_test/bucket+segments'),
-            ('PUT', '/v1/AUTH_test/bucket+segments/object/X'),
-        ], self.swift.calls)
+        if bucket_exists:
+            self.assertEqual([
+                ('HEAD', '/v1/AUTH_test/bucket'),
+                ('PUT', '/v1/AUTH_test/bucket+segments/object/X'),
+            ], self.swift.calls)
+        else:
+            self.assertEqual([
+                ('HEAD', '/v1/AUTH_test/bucket'),
+                ('PUT', '/v1/AUTH_test/bucket+segments'),
+                ('PUT', '/v1/AUTH_test/bucket+segments/object/X'),
+            ], self.swift.calls)
         self.swift.clear_calls()
 
-    def test_object_multipart_upload_initiate(self):
-        self._test_object_multipart_upload_initiate({})
-        self._test_object_multipart_upload_initiate({'Etag': 'blahblahblah'})
+    def test_object_multipart_upload_initiate_with_segment_bucket(self):
+        fake_memcache = FakeMemcache()
+        fake_memcache.store[get_cache_key(
+            'AUTH_test', 'bucket+segments')] = {'status': 204}
+        self._test_object_multipart_upload_initiate({}, fake_memcache)
+        self._test_object_multipart_upload_initiate({'Etag': 'blahblahblah'},
+                                                    fake_memcache)
         self._test_object_multipart_upload_initiate({
-            'Content-MD5': base64.b64encode('blahblahblahblah').strip()})
+            'Content-MD5': base64.b64encode(b'blahblahblahblah').strip()},
+            fake_memcache)
+
+    def test_object_multipart_upload_initiate_without_segment_bucket(self):
+        self.swift.register('PUT', '/v1/AUTH_test/bucket+segments',
+                            swob.HTTPCreated, {}, None)
+        fake_memcache = FakeMemcache()
+        fake_memcache.store[get_cache_key(
+            'AUTH_test', 'bucket+segments')] = {'status': 404}
+        self._test_object_multipart_upload_initiate({}, fake_memcache,
+                                                    bucket_exists=False)
+        self._test_object_multipart_upload_initiate({'Etag': 'blahblahblah'},
+                                                    fake_memcache,
+                                                    bucket_exists=False)
+        self._test_object_multipart_upload_initiate(
+            {'Content-MD5': base64.b64encode(b'blahblahblahblah').strip()},
+            fake_memcache,
+            bucket_exists=False)
 
-    @s3acl(s3acl_only=True)
     @patch('swift.common.middleware.s3api.controllers.multi_upload.'
            'unique_id', lambda: 'X')
-    def test_object_multipart_upload_initiate_s3acl(self):
+    def _test_object_multipart_upload_initiate_s3acl(
+            self, cache, should_head, should_put):
+        # mostly inlining stuff from @s3acl(s3_acl_only=True)
+        self.s3api.conf.s3_acl = True
+        self.swift.s3_acl = True
+        container_headers = encode_acl('container', ACL(
+            Owner('test:tester', 'test:tester'),
+            [Grant(User('test:tester'), 'FULL_CONTROL')]))
+        self.swift.register('HEAD', '/v1/AUTH_test/bucket',
+                            swob.HTTPNoContent, container_headers, None)
+        cache.store[get_cache_key('AUTH_test')] = {'status': 204}
+
         req = Request.blank('/bucket/object?uploads',
-                            environ={'REQUEST_METHOD': 'POST'},
+                            environ={'REQUEST_METHOD': 'POST',
+                                     'swift.cache': cache},
                             headers={'Authorization':
                                      'AWS test:tester:hmac',
                                      'Date': self.get_date_header(),
@@ -585,6 +626,16 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         status, headers, body = self.call_s3api(req)
         fromstring(body, 'InitiateMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
+        # Always need to check ACLs for the bucket
+        expected = [('HEAD', '/v1/AUTH_test/bucket')]
+        if should_head:
+            expected.append(('HEAD', '/v1/AUTH_test/bucket+segments'))
+        # XXX: For some reason there's always this second HEAD (???)
+        expected.append(('HEAD', '/v1/AUTH_test/bucket'))
+        if should_put:
+            expected.append(('PUT', '/v1/AUTH_test/bucket+segments'))
+        expected.append(('PUT', '/v1/AUTH_test/bucket+segments/object/X'))
+        self.assertEqual(expected, self.swift.calls)
 
         _, _, req_headers = self.swift.calls_with_headers[-1]
         self.assertEqual(req_headers.get('X-Object-Meta-Foo'), 'bar')
@@ -600,6 +651,31 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(acl_header.get(sysmeta_header('object', 'acl')),
                          tmpacl_header)
 
+    def test_object_multipart_upload_initiate_s3acl_with_segment_bucket(self):
+        self.swift.register('HEAD', '/v1/AUTH_test/bucket+segments',
+                            swob.HTTPNoContent, {}, None)
+        self._test_object_multipart_upload_initiate_s3acl(
+            FakeMemcache(), True, False)
+
+    def test_object_multipart_upload_initiate_s3acl_with_cached_seg_buck(self):
+        fake_memcache = FakeMemcache()
+        fake_memcache.store.update({
+            get_cache_key('AUTH_test', 'bucket+segments'): {'status': 204},
+        })
+        self._test_object_multipart_upload_initiate_s3acl(
+            fake_memcache, False, False)
+
+    def test_object_multipart_upload_initiate_s3acl_without_segment_bucket(
+            self):
+        fake_memcache = FakeMemcache()
+        fake_memcache.store.update({
+            get_cache_key('AUTH_test', 'bucket+segments'): {'status': 404},
+        })
+        self.swift.register('PUT', '/v1/AUTH_test/bucket+segments',
+                            swob.HTTPCreated, {}, None)
+        self._test_object_multipart_upload_initiate_s3acl(
+            fake_memcache, False, True)
+
     @s3acl(s3acl_only=True)
     @patch('swift.common.middleware.s3api.controllers.'
            'multi_upload.unique_id', lambda: 'X')
@@ -657,7 +733,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         with patch(
                 'swift.common.middleware.s3api.s3request.get_container_info',
                 lambda x, y: {'status': 404}):
@@ -667,15 +743,18 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(self._get_error_code(body), 'NoSuchBucket')
 
     def test_object_multipart_upload_complete(self):
+        content_md5 = base64.b64encode(hashlib.md5(
+            XML.encode('ascii')).digest())
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
-                                     'Date': self.get_date_header(), },
-                            body=xml)
+                                     'Date': self.get_date_header(),
+                                     'Content-MD5': content_md5, },
+                            body=XML)
         status, headers, body = self.call_s3api(req)
         elem = fromstring(body, 'CompleteMultipartUploadResult')
         self.assertNotIn('Etag', headers)
-        self.assertEqual(elem.find('ETag').text, s3_etag)
+        self.assertEqual(elem.find('ETag').text, S3_ETAG)
         self.assertEqual(status.split()[0], '200')
 
         self.assertEqual(self.swift.calls, [
@@ -694,10 +773,23 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(headers.get('X-Object-Meta-Foo'), 'bar')
         self.assertEqual(headers.get('Content-Type'), 'baz/quux')
         # SLO will provide a base value
-        override_etag = '; s3_etag=%s' % s3_etag.strip('"')
+        override_etag = '; s3_etag=%s' % S3_ETAG.strip('"')
         h = 'X-Object-Sysmeta-Container-Update-Override-Etag'
         self.assertEqual(headers.get(h), override_etag)
 
+    def test_object_multipart_upload_invalid_md5(self):
+        bad_md5 = base64.b64encode(hashlib.md5(
+            XML.encode('ascii') + b'some junk').digest())
+        req = Request.blank('/bucket/object?uploadId=X',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(),
+                                     'Content-MD5': bad_md5, },
+                            body=XML)
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual('400 Bad Request', status)
+        self.assertEqual(self._get_error_code(body), 'BadDigest')
+
     @patch('swift.common.middleware.s3api.controllers.multi_upload.time')
     def test_object_multipart_upload_complete_with_heartbeat(self, mock_time):
         self.swift.register(
@@ -708,16 +800,16 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             json.dumps([
                 {'name': item[0].replace('object', 'heartbeat-ok'),
                  'last_modified': item[1], 'hash': item[2], 'bytes': item[3]}
-                for item in objects_template
+                for item in OBJECTS_TEMPLATE
             ]))
         self.swift.register(
             'PUT', '/v1/AUTH_test/bucket/heartbeat-ok',
-            swob.HTTPAccepted, {}, [' ', ' ', ' ', json.dumps({
+            swob.HTTPAccepted, {}, [b' ', b' ', b' ', json.dumps({
                 'Etag': '"slo-etag"',
                 'Response Status': '201 Created',
                 'Errors': [],
-            })])
-        mock_time.return_value.time.side_effect = (
+            }).encode('ascii')])
+        mock_time.time.side_effect = (
             1,  # start_time
             12,  # first whitespace
             13,  # second...
@@ -732,16 +824,16 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
-        lines = body.split('\n')
-        self.assertTrue(lines[0].startswith('<?xml '))
+        lines = body.split(b'\n')
+        self.assertTrue(lines[0].startswith(b'<?xml '))
         self.assertTrue(lines[1])
         self.assertFalse(lines[1].strip())
         fromstring(body, 'CompleteMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
-        # NB: s3_etag includes quotes
-        self.assertIn('<ETag>%s</ETag>' % s3_etag, body)
+        # NB: S3_ETAG includes quotes
+        self.assertIn(('<ETag>%s</ETag>' % S3_ETAG).encode('ascii'), body)
         self.assertEqual(self.swift.calls, [
             ('HEAD', '/v1/AUTH_test/bucket'),
             ('HEAD', '/v1/AUTH_test/bucket+segments/heartbeat-ok/X'),
@@ -761,15 +853,15 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             json.dumps([
                 {'name': item[0].replace('object', 'heartbeat-fail'),
                  'last_modified': item[1], 'hash': item[2], 'bytes': item[3]}
-                for item in objects_template
+                for item in OBJECTS_TEMPLATE
             ]))
         self.swift.register(
             'PUT', '/v1/AUTH_test/bucket/heartbeat-fail',
-            swob.HTTPAccepted, {}, [' ', ' ', ' ', json.dumps({
+            swob.HTTPAccepted, {}, [b' ', b' ', b' ', json.dumps({
                 'Response Status': '400 Bad Request',
                 'Errors': [['some/object', '403 Forbidden']],
-            })])
-        mock_time.return_value.time.side_effect = (
+            }).encode('ascii')])
+        mock_time.time.side_effect = (
             1,  # start_time
             12,  # first whitespace
             13,  # second...
@@ -781,10 +873,10 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
-        lines = body.split('\n')
-        self.assertTrue(lines[0].startswith('<?xml '), (status, lines))
+        lines = body.split(b'\n')
+        self.assertTrue(lines[0].startswith(b'<?xml '), (status, lines))
         self.assertTrue(lines[1])
         self.assertFalse(lines[1].strip())
         fromstring(body, 'Error')
@@ -810,15 +902,15 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             json.dumps([
                 {'name': item[0].replace('object', 'heartbeat-fail'),
                  'last_modified': item[1], 'hash': item[2], 'bytes': item[3]}
-                for item in objects_template
+                for item in OBJECTS_TEMPLATE
             ]))
         self.swift.register(
             'PUT', '/v1/AUTH_test/bucket/heartbeat-fail',
-            swob.HTTPAccepted, {}, [' ', ' ', ' ', json.dumps({
+            swob.HTTPAccepted, {}, [b' ', b' ', b' ', json.dumps({
                 'Response Status': '400 Bad Request',
                 'Errors': [['some/object', '404 Not Found']],
-            })])
-        mock_time.return_value.time.side_effect = (
+            }).encode('ascii')])
+        mock_time.time.side_effect = (
             1,  # start_time
             12,  # first whitespace
             13,  # second...
@@ -830,10 +922,10 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
-        lines = body.split('\n')
-        self.assertTrue(lines[0].startswith('<?xml '))
+        lines = body.split(b'\n')
+        self.assertTrue(lines[0].startswith(b'<?xml '))
         self.assertTrue(lines[1])
         self.assertFalse(lines[1].strip())
         fromstring(body, 'Error')
@@ -856,7 +948,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '200')
         fromstring(body, 'CompleteMultipartUploadResult')
@@ -874,7 +966,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
         fromstring(body, 'CompleteMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
@@ -892,7 +984,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
         fromstring(body, 'CompleteMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
@@ -907,7 +999,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                                      'HTTP_HOST': 'localhost:8080:8080'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header(), },
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
         fromstring(body, 'CompleteMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
@@ -925,7 +1017,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             environ={'REQUEST_METHOD': 'POST'},
             headers={'Authorization': 'AWS test:tester:hmac',
                      'Date': self.get_date_header(), },
-            body=xml)
+            body=XML)
 
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '400')
@@ -946,7 +1038,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             environ={'REQUEST_METHOD': 'POST'},
             headers={'Authorization': 'AWS test:tester:hmac',
                      'Date': self.get_date_header(), },
-            body=xml)
+            body=XML)
 
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '400')
@@ -1102,8 +1194,8 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual(status.split()[0], '200')
         elem = fromstring(body, 'CompleteMultipartUploadResult')
         self.assertNotIn('Etag', headers)
-        expected_etag = '"%s-3"' % hashlib.md5(''.join(
-            x['hash'] for x in object_list).decode('hex')).hexdigest()
+        expected_etag = '"%s-3"' % hashlib.md5(binascii.unhexlify(''.join(
+            x['hash'] for x in object_list))).hexdigest()
         self.assertEqual(elem.find('ETag').text, expected_etag)
 
         self.assertEqual(self.swift.calls, [
@@ -1135,7 +1227,7 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header()},
-                            body=xml)
+                            body=XML)
         status, headers, body = self.call_s3api(req)
         fromstring(body, 'CompleteMultipartUploadResult')
         self.assertEqual(status.split()[0], '200')
@@ -1289,11 +1381,11 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         for p in elem.findall('Part'):
             partnum = int(p.find('PartNumber').text)
             self.assertEqual(p.find('LastModified').text,
-                             objects_template[partnum - 1][1][:-3] + 'Z')
+                             OBJECTS_TEMPLATE[partnum - 1][1][:-3] + 'Z')
             self.assertEqual(p.find('ETag').text.strip(),
-                             '"%s"' % objects_template[partnum - 1][2])
+                             '"%s"' % OBJECTS_TEMPLATE[partnum - 1][2])
             self.assertEqual(p.find('Size').text,
-                             str(objects_template[partnum - 1][3]))
+                             str(OBJECTS_TEMPLATE[partnum - 1][3]))
         self.assertEqual(status.split()[0], '200')
 
     def test_object_list_parts_encoding_type(self):
@@ -1378,11 +1470,11 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         for p in elem.findall('Part'):
             partnum = int(p.find('PartNumber').text)
             self.assertEqual(p.find('LastModified').text,
-                             objects_template[partnum - 1][1][:-3] + 'Z')
+                             OBJECTS_TEMPLATE[partnum - 1][1][:-3] + 'Z')
             self.assertEqual(p.find('ETag').text,
-                             '"%s"' % objects_template[partnum - 1][2])
+                             '"%s"' % OBJECTS_TEMPLATE[partnum - 1][2])
             self.assertEqual(p.find('Size').text,
-                             str(objects_template[partnum - 1][3]))
+                             str(OBJECTS_TEMPLATE[partnum - 1][3]))
         self.assertEqual(status.split()[0], '200')
 
     def test_object_list_parts_over_max_32bit_int(self):
@@ -1573,21 +1665,21 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
     def test_complete_multipart_upload_acl_without_permission(self):
         status, headers, body = \
             self._test_for_s3acl('POST', '?uploadId=X', 'test:other',
-                                 body=xml)
+                                 body=XML)
         self.assertEqual(status.split()[0], '403')
 
     @s3acl(s3acl_only=True)
     def test_complete_multipart_upload_acl_with_write_permission(self):
         status, headers, body = \
             self._test_for_s3acl('POST', '?uploadId=X', 'test:write',
-                                 body=xml)
+                                 body=XML)
         self.assertEqual(status.split()[0], '200')
 
     @s3acl(s3acl_only=True)
     def test_complete_multipart_upload_acl_with_fullcontrol_permission(self):
         status, headers, body = \
             self._test_for_s3acl('POST', '?uploadId=X', 'test:full_control',
-                                 body=xml)
+                                 body=XML)
         self.assertEqual(status.split()[0], '200')
 
     def _test_copy_for_s3acl(self, account, src_permission=None,
@@ -1634,6 +1726,16 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         _, _, headers = self.swift.calls_with_headers[-1]
         self.assertEqual(headers['X-Copy-From'], '/src_bucket/src_obj')
         self.assertEqual(headers['Content-Length'], '0')
+        # Some headers *need* to get cleared in case we're copying from
+        # another multipart upload
+        for header in (
+            'X-Object-Sysmeta-S3api-Etag',
+            'X-Object-Sysmeta-Slo-Etag',
+            'X-Object-Sysmeta-Slo-Size',
+            'X-Object-Sysmeta-Container-Update-Override-Etag',
+            'X-Object-Sysmeta-Swift3-Etag',
+        ):
+            self.assertEqual(headers[header], '')
 
     @s3acl(s3acl_only=True)
     def test_upload_part_copy_acl_with_owner_permission(self):
@@ -1829,8 +1931,8 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
             account, src_headers={'Content-Length': '10'}, put_header=header)
 
         self.assertEqual(status.split()[0], '400')
-        self.assertIn('Range specified is not valid for '
-                      'source object of size: 10', body)
+        self.assertIn(b'Range specified is not valid for '
+                      b'source object of size: 10', body)
 
         self.assertEqual([
             ('HEAD', '/v1/AUTH_test/bucket'),
@@ -1873,9 +1975,9 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
         self.assertEqual('/src_bucket/src_obj', put_headers['X-Copy-From'])
 
     def _test_no_body(self, use_content_length=False,
-                      use_transfer_encoding=False, string_to_md5=''):
+                      use_transfer_encoding=False, string_to_md5=b''):
         raw_md5 = hashlib.md5(string_to_md5).digest()
-        content_md5 = raw_md5.encode('base64').strip()
+        content_md5 = base64.b64encode(raw_md5).strip()
         with UnreadableInput(self) as fake_input:
             req = Request.blank(
                 '/bucket/object?uploadId=X',
@@ -1900,11 +2002,11 @@ class TestS3ApiMultiUpload(S3ApiTestCase):
     @s3acl
     def test_object_multi_upload_empty_body(self):
         self._test_no_body()
-        self._test_no_body(string_to_md5='test')
+        self._test_no_body(string_to_md5=b'test')
         self._test_no_body(use_content_length=True)
-        self._test_no_body(use_content_length=True, string_to_md5='test')
+        self._test_no_body(use_content_length=True, string_to_md5=b'test')
         self._test_no_body(use_transfer_encoding=True)
-        self._test_no_body(use_transfer_encoding=True, string_to_md5='test')
+        self._test_no_body(use_transfer_encoding=True, string_to_md5=b'test')
 
 
 class TestS3ApiMultiUploadNonUTC(TestS3ApiMultiUpload):
