@@ -32,21 +32,26 @@ class TestHeaderEnvironProxy(unittest.TestCase):
     def test_proxy(self):
         environ = {}
         proxy = swift.common.swob.HeaderEnvironProxy(environ)
+        self.assertIs(environ, proxy.environ)
         proxy['Content-Length'] = 20
         proxy['Content-Type'] = 'text/plain'
         proxy['Something-Else'] = 'somevalue'
-        self.assertEqual(
-            proxy.environ, {'CONTENT_LENGTH': '20',
-                            'CONTENT_TYPE': 'text/plain',
-                            'HTTP_SOMETHING_ELSE': 'somevalue'})
+        # NB: WSGI strings
+        proxy['X-Object-Meta-Unicode-\xff-Bu\xc3\x9fe'] = '\xe2\x98\xb9'
+        self.assertEqual(proxy.environ, {
+            'CONTENT_LENGTH': '20',
+            'CONTENT_TYPE': 'text/plain',
+            'HTTP_SOMETHING_ELSE': 'somevalue',
+            'HTTP_X_OBJECT_META_UNICODE_\xff_BU\xc3\x9fE': '\xe2\x98\xb9'})
         self.assertEqual(proxy['content-length'], '20')
         self.assertEqual(proxy['content-type'], 'text/plain')
         self.assertEqual(proxy['something-else'], 'somevalue')
         self.assertEqual(set(['Something-Else',
+                              'X-Object-Meta-Unicode-\xff-Bu\xc3\x9fE',
                               'Content-Length', 'Content-Type']),
                          set(proxy.keys()))
         self.assertEqual(list(iter(proxy)), proxy.keys())
-        self.assertEqual(3, len(proxy))
+        self.assertEqual(4, len(proxy))
 
     def test_ignored_keys(self):
         # Constructor doesn't normalize keys
@@ -58,6 +63,8 @@ class TestHeaderEnvironProxy(unittest.TestCase):
         self.assertEqual(0, len(proxy))
         self.assertRaises(KeyError, proxy.__getitem__, key)
         self.assertNotIn(key, proxy)
+        self.assertIn(key, proxy.environ)
+        self.assertIs(environ, proxy.environ)
 
         proxy['Content-Type'] = 'text/plain'
         self.assertEqual(['Content-Type'], list(iter(proxy)))
@@ -77,6 +84,8 @@ class TestHeaderEnvironProxy(unittest.TestCase):
         del proxy['Something-Else']
         self.assertEqual(proxy.environ, {})
         self.assertEqual(0, len(proxy))
+        with self.assertRaises(KeyError):
+            del proxy['Content-Length']
 
     def test_contains(self):
         environ = {}
@@ -670,7 +679,7 @@ class TestRequest(unittest.TestCase):
 
         def test_app(environ, start_response):
             start_response('401 Unauthorized', [])
-            return ['hi']
+            return [b'hi']
 
         # Request environment contains valid account in path
         req = swift.common.swob.Request.blank('/v1/account-name')
@@ -692,7 +701,7 @@ class TestRequest(unittest.TestCase):
 
         def test_app(environ, start_response):
             start_response('401 Unauthorized', [])
-            return ['hi']
+            return [b'hi']
 
         # Request environment contains bad path
         req = swift.common.swob.Request.blank('/random')
@@ -706,7 +715,7 @@ class TestRequest(unittest.TestCase):
 
         def test_app(environ, start_response):
             start_response('401 Unauthorized', [])
-            return ['no creds in request']
+            return [b'no creds in request']
 
         # Request to get token
         req = swift.common.swob.Request.blank('/v1.0/auth')
@@ -729,7 +738,7 @@ class TestRequest(unittest.TestCase):
         def test_app(environ, start_response):
             start_response('401 Unauthorized', {
                            'Www-Authenticate': 'Me realm="whatever"'})
-            return ['no creds in request']
+            return [b'no creds in request']
 
         # Auth middleware sets own Www-Authenticate
         req = swift.common.swob.Request.blank('/auth/v1.0')
@@ -743,7 +752,7 @@ class TestRequest(unittest.TestCase):
 
         def test_app(environ, start_response):
             start_response('401 Unauthorized', [])
-            return ['hi']
+            return [b'hi']
 
         hacker = 'account-name\n\n<b>foo<br>'  # url injection test
         quoted_hacker = quote(hacker)
@@ -766,7 +775,7 @@ class TestRequest(unittest.TestCase):
         # Other status codes should not have WWW-Authenticate in response
         def test_app(environ, start_response):
             start_response('200 OK', [])
-            return ['hi']
+            return [b'hi']
 
         req = swift.common.swob.Request.blank('/')
         resp = req.get_response(test_app)
@@ -1626,11 +1635,6 @@ class TestResponse(unittest.TestCase):
         self.assertEqual(resp.body, b'')
 
 
-class TestUTC(unittest.TestCase):
-    def test_tzname(self):
-        self.assertEqual(swift.common.swob.UTC.tzname(None), 'UTC')
-
-
 class TestConditionalIfNoneMatch(unittest.TestCase):
     def fake_app(self, environ, start_response):
         start_response('200 OK', [('Etag', 'the-etag')])
@@ -1763,7 +1767,7 @@ class TestConditionalIfMatch(unittest.TestCase):
 
         def fake_app_404(environ, start_response):
             start_response('404 Not Found', [])
-            return ['hi']
+            return [b'hi']
 
         req = swift.common.swob.Request.blank(
             '/', headers={'If-Match': '*'})
