@@ -29,10 +29,7 @@ import eventlet.wsgi
 
 import six
 from six import BytesIO
-from six import StringIO
 from six.moves.urllib.parse import quote
-if six.PY2:
-    import mimetools
 
 import mock
 
@@ -69,57 +66,11 @@ def _fake_rings(tmpdir):
 class TestWSGI(unittest.TestCase):
     """Tests for swift.common.wsgi"""
 
-    def setUp(self):
-        if six.PY2:
-            self._orig_parsetype = mimetools.Message.parsetype
-
-    def tearDown(self):
-        if six.PY2:
-            mimetools.Message.parsetype = self._orig_parsetype
-
-    @unittest.skipIf(six.PY3, "test specific to Python 2")
-    def test_monkey_patch_mimetools(self):
-        sio = StringIO('blah')
-        self.assertEqual(mimetools.Message(sio).type, 'text/plain')
-        sio = StringIO('blah')
-        self.assertEqual(mimetools.Message(sio).plisttext, '')
-        sio = StringIO('blah')
-        self.assertEqual(mimetools.Message(sio).maintype, 'text')
-        sio = StringIO('blah')
-        self.assertEqual(mimetools.Message(sio).subtype, 'plain')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).type, 'text/html')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).plisttext,
-                         '; charset=ISO-8859-4')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).maintype, 'text')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).subtype, 'html')
-
-        wsgi.monkey_patch_mimetools()
-        sio = StringIO('blah')
-        self.assertIsNone(mimetools.Message(sio).type)
-        sio = StringIO('blah')
-        self.assertEqual(mimetools.Message(sio).plisttext, '')
-        sio = StringIO('blah')
-        self.assertIsNone(mimetools.Message(sio).maintype)
-        sio = StringIO('blah')
-        self.assertIsNone(mimetools.Message(sio).subtype)
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).type, 'text/html')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).plisttext,
-                         '; charset=ISO-8859-4')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).maintype, 'text')
-        sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEqual(mimetools.Message(sio).subtype, 'html')
-
     def test_init_request_processor(self):
         config = """
         [DEFAULT]
         swift_dir = TEMPDIR
+        fallocate_reserve = 1%
 
         [pipeline:main]
         pipeline = proxy-server
@@ -172,6 +123,7 @@ class TestWSGI(unittest.TestCase):
             '__file__': conf_file,
             'here': os.path.dirname(conf_file),
             'conn_timeout': '0.2',
+            'fallocate_reserve': '1%',
             'swift_dir': t,
             '__name__': 'proxy-server'
         }
@@ -830,25 +782,34 @@ class TestWSGI(unittest.TestCase):
                 'logger',
                 'log_name')
 
+        loadapp_conf = []
+        to_inject = object()  # replication_timeout injects non-string data
+
         def _global_conf_callback(preloaded_app_conf, global_conf):
             calls['_global_conf_callback'] += 1
             self.assertEqual(
                 preloaded_app_conf, {'__file__': 'test', 'workers': 0})
             self.assertEqual(global_conf, {'log_name': 'log_name'})
-            global_conf['test1'] = 'one'
+            global_conf['test1'] = to_inject
 
         def _loadapp(uri, name=None, **kwargs):
             calls['_loadapp'] += 1
-            self.assertTrue('global_conf' in kwargs)
-            self.assertEqual(kwargs['global_conf'],
-                             {'log_name': 'log_name', 'test1': 'one'})
+            self.assertIn('global_conf', kwargs)
+            loadapp_conf.append(kwargs['global_conf'])
+            # global_conf_callback hasn't been called yet
+            self.assertNotIn('test1', kwargs['global_conf'])
+
+        def _run_server(*args, **kwargs):
+            # but by the time that we actually *run* the server, it has
+            self.assertEqual(loadapp_conf,
+                             [{'log_name': 'log_name', 'test1': to_inject}])
 
         with mock.patch.object(wsgi, '_initrp', _initrp), \
                 mock.patch.object(wsgi, 'get_socket'), \
                 mock.patch.object(wsgi, 'drop_privileges'), \
                 mock.patch.object(wsgi, 'loadapp', _loadapp), \
                 mock.patch.object(wsgi, 'capture_stdio'), \
-                mock.patch.object(wsgi, 'run_server'), \
+                mock.patch.object(wsgi, 'run_server', _run_server), \
                 mock.patch('swift.common.utils.eventlet') as _utils_evt:
             wsgi.run_wsgi('conf_file', 'app_section',
                           global_conf_callback=_global_conf_callback)
@@ -1035,11 +996,6 @@ class TestWSGI(unittest.TestCase):
 
 
 class TestSwiftHttpProtocol(unittest.TestCase):
-    def setUp(self):
-        patcher = mock.patch('swift.common.wsgi.wsgi.HttpProtocol')
-        self.mock_super = patcher.start()
-        self.addCleanup(patcher.stop)
-
     def _proto_obj(self):
         # Make an object we can exercise... note the base class's __init__()
         # does a bunch of work, so we just new up an object like eventlet.wsgi
@@ -1092,12 +1048,38 @@ class TestSwiftHttpProtocol(unittest.TestCase):
 
         self.assertEqual(False, proto_obj.parse_request())
 
-        self.assertEqual([], self.mock_super.mock_calls)
         self.assertEqual([
             mock.call(400, "Bad HTTP/0.9 request type ('jimmy')"),
         ], proto_obj.send_error.mock_calls)
         self.assertEqual(('a', '123'), proto_obj.client_address)
 
+    def test_request_line_cleanup(self):
+        def do_test(line_from_socket, expected_line=None):
+            if expected_line is None:
+                expected_line = line_from_socket
+
+            proto_obj = self._proto_obj()
+            proto_obj.raw_requestline = line_from_socket
+            with mock.patch('swift.common.wsgi.wsgi.HttpProtocol') \
+                    as mock_super:
+                proto_obj.parse_request()
+
+            self.assertEqual([mock.call.parse_request(proto_obj)],
+                             mock_super.mock_calls)
+            self.assertEqual(proto_obj.raw_requestline, expected_line)
+
+        do_test(b'GET / HTTP/1.1')
+        do_test(b'GET /%FF HTTP/1.1')
+
+        if not six.PY2:
+            do_test(b'GET /\xff HTTP/1.1', b'GET /%FF HTTP/1.1')
+            do_test(b'PUT /Here%20Is%20A%20SnowMan:\xe2\x98\x83 HTTP/1.0',
+                    b'PUT /Here%20Is%20A%20SnowMan%3A%E2%98%83 HTTP/1.0')
+            do_test(
+                b'POST /?and%20it=fixes+params&'
+                b'PALMTREE=\xf0%9f\x8c%b4 HTTP/1.1',
+                b'POST /?and+it=fixes+params&PALMTREE=%F0%9F%8C%B4 HTTP/1.1')
+
 
 class TestProxyProtocol(unittest.TestCase):
     def _run_bytes_through_protocol(self, bytes_from_client, protocol_class):
@@ -2084,6 +2066,7 @@ class TestPipelineModification(unittest.TestCase):
 
             [filter:tempauth]
             use = egg:swift#tempauth
+            user_test_tester = t%%sting .admin
 
             [filter:copy]
             use = egg:swift#copy
@@ -2110,7 +2093,10 @@ class TestPipelineModification(unittest.TestCase):
             for version, pipeline, expected in to_test:
                 conf_file = os.path.join(t, 'proxy-server.conf')
                 with open(conf_file, 'w') as f:
-                    f.write(contents % (t, pipeline))
+                    to_write = contents % (t, pipeline)
+                    # Sanity check that the password only has one % in it
+                    self.assertIn('t%sting', to_write)
+                    f.write(to_write)
                 app = wsgi.loadapp(conf_file, global_conf={})
 
                 actual = ' '.join(m.rsplit('.', 1)[1]
